# To be called with full path of this script!

use strict "vars";

# Perl script to collect SCC data in snapshot and produce logbook and convert to html-format.
# Copyright (C) 2006-2009 QNH.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; see the file COPYING.
# If not, write to the Free Software Foundation,
# 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
# Contact information: http://scc.qnh-infrastructure.nl

# SCC-release:	1.1.5
# file-version:	$Revision: 1.27 $


my $PROGNAME = "$0";

if ( $ENV{ TRACE } > 0 )
{
	print STDERR "start $PROGNAME \n";
}

my $SYNTAX = "Syntax error, use: $PROGNAME [ --extra <mod> ] [ --timeout <duration> ] <name>\n";

my $extra_mod = "";
my $timeout = 0;
while ( @ARGV )
{
	if ( "$ARGV[ 0 ]" =~ /^--extra$/ )
	{
		shift( @ARGV );
		if ( @ARGV or die "$SYNTAX" )
		{
			$extra_mod = "$ARGV[ 0 ]";
		}
		shift( @ARGV );
	}
	elsif ( "$ARGV[ 0 ]" =~ /^--timeout$/ )
	{
		shift( @ARGV );
		if ( @ARGV or die "$SYNTAX" )
		{
			$timeout = "$ARGV[ 0 ]";
		}
		shift( @ARGV );
	}
	elsif ( "$ARGV[ 0 ]" =~ /^--/ )
	{
		die "$SYNTAX";
	}
	else
	{
		last;		# Hostname is recorded in variable outside this loop
	}
}

# Determine our directories.
my $dir = $0;
$dir =~ s/[^\\]*$//;		# Erase the name of the script.
$dir =~ s/[^\\]*\\$//;		# Erase deepest directory (bin)

if ( $timeout > 0 )
{
	# Indicate timeout when the alarm hits.
	$SIG{ALRM} = sub { print STDERR "$PROGNAME: timeout, program aborted\n"; exit 2 };

	alarm( $timeout );
}

my $SCC_DATA	= $dir		. "data\\";
my $SCC_TMP	= $dir		. "tmp\\";
my $SCC_BIN	= $dir		. "bin\\";
my $SCC_MODULES	= $SCC_BIN	. "scc_modules\\";

$ENV{ SCC_BIN } = $SCC_BIN;
$ENV{ SCC_TMP } = $SCC_TMP;
$ENV{ SCC_DATA } = $SCC_DATA;

chdir( $SCC_TMP ) or die "$PROGNAME: Cannot change directory to $SCC_TMP";

my $new_style_sheet = $dir . "doc\\style.css";
my $cur_style_sheet = $SCC_DATA . "style.css";

# When no stylesheet is present, we copy our version.
# This way we will not overwrite any changes the admin has made.
if ( ! -f $cur_style_sheet )
{
	system( "copy", "\"$new_style_sheet\"", "\"$cur_style_sheet\"" );
}

my $tmp_1_file = "scc_log_1.txt";

my $remark = "";
my $restart = 0;

my $hostname = "\L$ARGV[ 0 ]";

my $computername = $hostname;
$computername =~ s/\..*//;			# Do not use the (dns) domain for WMI.
$ENV{ SCC_WMI_COMPUTER } = "$computername";

my $cur_snapshot	= $SCC_DATA . "scc." . $hostname . ".cur";
my $old_snapshot	= $SCC_DATA . "scc." . $hostname . ".old";
my $fix_cur		= $SCC_TMP  . "scc." . $hostname . "_fix.cur";
my $new_snapshot	= $SCC_TMP  . "scc." . $hostname . ".cur";
my $tmp_snapshot	= $SCC_DATA . "scc." . $hostname . ".tmp";
my $cur_snap_html	= $SCC_DATA . "scc." . $hostname . ".html";
my $fix_new		= $SCC_TMP  . "scc." . $hostname . "_fix.new";

my $cur_log		= $SCC_DATA . "scc." . $hostname . ".log";
my $cur_log_html	= $SCC_DATA . "scc." . $hostname . ".log.html";

my $index_html		= $SCC_DATA . "index.html";

my $diff = "\"$SCC_BIN" . "diff\"";

my $mytime = "";
my $date = "";
my $timestamp = "";
my $runtime = 0;

if ( -f $cur_snapshot )
{
	$restart = 1;

	open VERSION,"<$cur_snapshot";
	while ( <VERSION> )
	{
		chomp;
		# Refer to the code of the general module for the exact string!
		if ( "$_" =~ /^fix:general::layout-version:1.1.5$/ )
		{
			# Same version, do not indicate restart.
			$restart = 0;
			last;			# No need to check more data.
		}
	}
	close VERSION;
}

open STDOUT,">$new_snapshot";

my $sec = 0;
my $min = 0;
my $hour = 0;
my $year = 0;
my $month = 0;
my $day = 0;
my $dummy;

my $now = time();
($sec,$min,$hour,$day,$month,$year,$dummy,$dummy,$dummy) = localtime( $now );
printf "var:general::date:%04d-%02d-%02d\n", 1900 + $year, 1 + $month, $day;
printf "var:general::start time:%02d.%02d.%02d\n", $hour, $min, $sec ;
$timestamp = sprintf( "%04d-%02d-%02d:%02d.%02d.%02d", 1900 + $year, 1 + $month, $day, $hour, $min, $sec );

my $start_tick = 3600 * $hour + 60 * $min + $sec;

# Collect the data.
my $collect = $SCC_BIN . "scc-collect.pl";
my @args = ( "perl", "\"$collect\"" );
if ( $extra_mod ne "")
{
	@args = ( @args, "--extra", "$extra_mod" );
}
if ( system( @args ) )
{
	# Fatal error during collection of data.
	close STDOUT;
	unlink $new_snapshot;

	print STDERR "$PROGNAME: unable to collect data for $hostname : $! \n";
	exit 1;				# Indicate failure.
}


my $now = time();
($sec,$min,$hour,$day,$month,$year,$dummy,$dummy,$dummy) = localtime( $now );
printf "var:general::stop time:%02d.%02d.%02d\n", $hour, $min, $sec;

my $end_tick = 3600 * $hour + 60 * $min + $sec;

if ( $end_tick < $start_tick )
{
	# Ended another day than we started; correction with 24 hours.
	$end_tick += 24 * 3600;
}
my $runtime = $end_tick - $start_tick;
print "var:general::runtime:$runtime\n";

close STDOUT;

# Check the syntax of the snapshot.
my %visited = ();
my %compare = ();
my $syntax_errors = 0;

open DATA,"<$new_snapshot";
open SNAP,">$tmp_snapshot";
while ( <DATA> )
{
	if ( $_ =~ /^fix:messages:/ )
	{
		# When a module signals an error, the syntax of the message-line is correct
		$visited{ "fix:messages::" } = 1;
		$compare{ ":messages:" } = 1;
	}

	if ( ( $_ =~ /^fix:/ ) || ( $_ =~ /^var:/ ) || ( $_ =~ /^hlp:/ ) )
	{
		my $class = $_;
		chomp( $class );
		my $line = $class;
		$class =~ s/::.*/::/;
		my @part = split( /:/, $class );
		my $cnt = $#part;

		if ( $visited{ $class } )	# Each visited class already has been checked.
		{
			print SNAP $_;		# print, but do not check again.
			next;
		}

		# line with classification with minimal number of elements:
		#	fix:general::			# cnt is 1
		# line with classification with maximal number of elements:
		#	fix:1:2:3:4:5:6:7:8:9:10::	# cnt is 10
		if ( $cnt < 1 )
		{
			print SNAP "fix:messages::not enough fields in classification: " . $_;
			$syntax_errors = 1;
		}
		elsif ( $cnt > 10 )
		{
			print SNAP "fix:messages::too many fields in classification: " . $_;
			$syntax_errors = 1;
		}
		elsif ( ( $class eq $line ) && ( ! ( $class =~ /::$/ ) ) )
		{
			print SNAP "fix:messages::no end of classification: " . $_;
			$syntax_errors = 1;
		}
		else
		{
			$visited{ $class } = 1;

			# A class conflict occurs in the following case:
			# fix:a:b::data
			# fix:a:b:c::data
			# Either a:b is the classification of configuration data,
			# or it is the entry-point for a sub-menu.
			# Help-info can be available for a sub-menu.
			# Therefore no class conflicts for help-info.
			if ( ! ( $class =~ /^hlp:/ ) )
			{
				$class =~ s/^...//;		# erase fix or var or hlp
				$class =~ s/::$/:/;		# map :a:: to :a: to check for :a:b:
				foreach my $c ( %compare )
				{
					if ( $c eq $class )
					{
						last;
					}
					my $label = "fix:messages::class conflict for";
					if ( index( $c, $class ) == 0 )
					{
						printf( SNAP "$label \"%s\" and \"%s\"\n", $class, $c );
						$syntax_errors = 1;
						last;
					}
					if ( index( $class, $c ) == 0 )
					{
						printf( SNAP "$label \"%s\" and \"%s\"\n", $c, $class );
						$syntax_errors = 1;
						last;
					}
				}
				$compare{ $class } = 1;
			}
			print SNAP $_;
		}
	}
	else
	{
		print SNAP "fix:messages::unknown prefix: " . $_;
	}
}
close DATA;
close SNAP;

if ( $syntax_errors > 0 )
{
	# Snapshot contains syntax errors that have to be reported on the first line of the snapshot.
	open DATA,"<$tmp_snapshot";
	open SNAP,">$new_snapshot";
	print SNAP "fix:messages::inspect scc.$hostname" . ".cur to determine cause of messages in system/user-modules\n";
	while ( <DATA> )
	{
		print SNAP $_;			# Copy the data including reported errors.
	}
	close DATA;
	close SNAP;
}
unlink( $tmp_snapshot );		# not needed anymore.

if ( $restart > 0 )
{
	unlink $cur_snapshot;
	$remark = "install of SCC release 1.1.5";
}

if ( -f "$cur_snapshot" )
{
	# Get the fix lines of the previous snapshot (cur-file)
	open STDIN,"<$cur_snapshot";
	open STDOUT,">$fix_cur";
	while ( <STDIN> )
	{
		if ( $_ =~ "^fix:" )
		{
			print $_;
		}
	}
	close STDOUT;
	close STDIN;

	# Get the fix lines of the new snapshot (new-file)
	open STDIN,"<$new_snapshot";
	open STDOUT,">$fix_new";
	while ( <STDIN> )
	{
		if ( $_ =~ "^fix:" )
		{
			print $_;
		}
	}
	close STDOUT;
	close STDIN;

	my $diff_new = "$SCC_TMP" . "scc_diff.new";
	# We cannot check the exit status of diff as it indicates the presence of differences.
	# The absolte path for diff did not work for some systems. 
	my $sys_cmd = sprintf( "..\\bin\\diff \"$fix_cur\" \"$fix_new\" >\"$diff_new\"" );
	system( $sys_cmd );

	open STDOUT,">>$cur_log";
	if ( -z $diff_new )
	{
		print "$timestamp:result::identical\n";
		print "$timestamp:runtime::$runtime\n";
	}
	else
	{
		print "$timestamp:result::different\n";
		print "$timestamp:runtime::$runtime\n";

		# Determine the number of differences.
		my $count = 0;
		open STDIN,"<$diff_new";
		while ( <STDIN> )
		{
			if ( $_ =~ "^[><]" )
			{
				$count++;
			}
		}
		close STDIN;
		print "$timestamp:count::$count\n";

		# Determine the start-date and time of the previous run.
		my $prev_date;
		my $prev_time;
		open STDIN,"<$cur_snapshot";
		while ( <STDIN> )
		{
			chomp;
			if ( $_ =~ "::date:" )
			{
				s/.*://g;
				$prev_date = $_;
			}
			if ( $_ =~ "::start time:" )
			{
				s/.*://g;
				$prev_time = $_;
				last;
			}
		}
		close STDIN;
		print "$timestamp:previous date::$prev_date\n";
		print "$timestamp:previous time::$prev_time\n";

		# Display the differences
		open STDIN,"<$diff_new";
		while ( <STDIN> )
		{
			chomp;

			if ( $_ =~ "^<" )
			{
				s/<.fix:/old::/;
				print "$timestamp:data::$_\n";
			}
			if ( $_ =~ "^>" )
			{
				s/>.fix:/new::/;
				print "$timestamp:data::$_\n";
			}
		}
		close STDIN;
	}
	close STDOUT;
	unlink $diff_new;
}
else
{
	open STDOUT,">>$cur_log";
	print "$timestamp:result::(re)start\n";
	print "$timestamp:remark::$remark\n";
	print "$timestamp:runtime::$runtime\n";
	close STDOUT;
}

unlink $fix_cur;
unlink $fix_new;

# Replace old-snapshot with current (previous) and current (previous) with new. 
unlink $old_snapshot;
rename $cur_snapshot, $old_snapshot;
rename $new_snapshot, $cur_snapshot;

# New snapshot is current file, produce (new) current html file from snapshot.
open STDIN,"<$cur_snapshot" or die "$PROGNAME: Unable to read snapshot";
open STDOUT,">$cur_snap_html" or die "$PROGNAME: Unable to write html";
my $snap = $SCC_BIN . "scc-snap2html.pl";
if ( system( ( "perl", "\"$snap\"", "$hostname" ) ) )
{
	print STDERR "$PROGNAME: unable to convert snapshot to HTML: $! \n";
	exit 1;				# Indicate failure.
}
close STDOUT;
close STDIN;

open STDIN,"<$cur_log";
open STDOUT,">$cur_log_html";
my $log = $SCC_BIN . "scc-log2html.pl";
if ( system( ( "perl", "\"$log\"", "$hostname" ) ) )
{
	print STDERR "$PROGNAME: unable to convert logbook to HTML for $hostname : $! \n";
	exit 1;				# Indicate failure.
}
close STDIN;
close STDOUT;

if ( $ENV{ TRACE } > 0 )
{
	print STDERR "end $PROGNAME \n";
}

my $timeleft = alarm( 0 ); 		# Reset alarm timer. All processing, except for the exit, is done.

exit 0 ;
